package org.eclipse.jdt.internal.compiler.apt.util;

package .tmp;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CoderResult;
import java.nio.charset.CodingErrorAction;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;
import javax.tools.FileObject;

public final class Util {
  public static String LINE_SEPARATOR = System.getProperty("line.separator");
  
  public static char[] getInputStreamAsCharArray(InputStream stream, int length, String encoding) throws IOException {
    Charset charset = null;
    try {
      charset = Charset.forName(encoding);
    } catch (IllegalCharsetNameException illegalCharsetNameException) {
      System.err.println("Illegal charset name : " + encoding);
      return null;
    } catch (UnsupportedCharsetException unsupportedCharsetException) {
      System.err.println("Unsupported charset : " + encoding);
      return null;
    } 
    CharsetDecoder charsetDecoder = charset.newDecoder();
    charsetDecoder.onMalformedInput(CodingErrorAction.REPLACE).onUnmappableCharacter(CodingErrorAction.REPLACE);
    byte[] contents = org.eclipse.jdt.internal.compiler.util.Util.getInputStreamAsByteArray(stream, length);
    ByteBuffer byteBuffer = ByteBuffer.allocate(contents.length);
    byteBuffer.put(contents);
    byteBuffer.flip();
    return charsetDecoder.decode(byteBuffer).array();
  }
  
  public static CharSequence getCharContents(FileObject fileObject, boolean ignoreEncodingErrors, byte[] contents, String encoding) throws IOException {
    if (contents == null)
      return null; 
    Charset charset = null;
    try {
      charset = Charset.forName(encoding);
    } catch (IllegalCharsetNameException illegalCharsetNameException) {
      System.err.println("Illegal charset name : " + encoding);
      return null;
    } catch (UnsupportedCharsetException unsupportedCharsetException) {
      System.err.println("Unsupported charset : " + encoding);
      return null;
    } 
    CharsetDecoder charsetDecoder = charset.newDecoder();
    ByteBuffer byteBuffer = ByteBuffer.allocate(contents.length);
    byteBuffer.put(contents);
    byteBuffer.flip();
    if (ignoreEncodingErrors) {
      charsetDecoder.onMalformedInput(CodingErrorAction.REPLACE).onUnmappableCharacter(CodingErrorAction.REPLACE);
      return charsetDecoder.decode(byteBuffer);
    } 
    charsetDecoder.onMalformedInput(CodingErrorAction.REPORT).onUnmappableCharacter(CodingErrorAction.REPORT);
    CharBuffer out = CharBuffer.allocate(contents.length);
    CoderResult result = null;
    String replacement = charsetDecoder.replacement();
    int replacementLength = replacement.length();
    EncodingErrorCollector collector = null;
    while (true) {
      result = charsetDecoder.decode(byteBuffer, out, true);
      if (result.isMalformed() || result.isUnmappable()) {
        if (collector == null)
          collector = new EncodingErrorCollector(fileObject, encoding); 
        reportEncodingError(collector, out.position(), result.length());
        if (out.position() + replacementLength >= out.capacity()) {
          CharBuffer temp = CharBuffer.allocate(out.capacity() * 2);
          out.flip();
          temp.put(out);
          out = temp;
        } 
        out.append(replacement);
        byteBuffer.position(byteBuffer.position() + result.length());
        continue;
      } 
      if (result.isOverflow()) {
        CharBuffer temp = CharBuffer.allocate(out.capacity() * 2);
        out.flip();
        temp.put(out);
        out = temp;
        continue;
      } 
      break;
    } 
    out.flip();
    if (collector != null)
      collector.reportAllEncodingErrors(out.toString()); 
    return out;
  }
  
  private static void reportEncodingError(EncodingErrorCollector collector, int position, int length) {
    collector.collect(position, -length);
  }
}
